"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandsObject = void 0;
const tslib_1 = require("tslib");
const enquirer = require("enquirer");
const yargs = require("yargs");
const chalk = require("chalk");
const ci_list_1 = require("../src/utils/ci/ci-list");
const create_workspace_1 = require("../src/create-workspace");
const preset_1 = require("../src/utils/preset/preset");
const preset_options_1 = require("../src/utils/preset/preset-options");
const ab_testing_1 = require("../src/utils/nx/ab-testing");
const output_1 = require("../src/utils/output");
const default_base_1 = require("../src/utils/git/default-base");
const string_utils_1 = require("../src/utils/string-utils");
const package_manager_1 = require("../src/utils/package-manager");
const nx_version_1 = require("../src/utils/nx/nx-version");
const point_to_tutorial_and_course_1 = require("../src/utils/preset/point-to-tutorial-and-course");
const decorator_1 = require("./decorator");
const get_third_party_preset_1 = require("../src/utils/preset/get-third-party-preset");
const framework_list_1 = require("./types/framework-list");
const bundler_list_1 = require("./types/bundler-list");
exports.commandsObject = yargs
    .wrap(yargs.terminalWidth())
    .parserConfiguration({
    'strip-dashed': true,
    'dot-notation': true,
})
    .command(
// this is the default and only command
'$0 [name] [options]', 'Create a new Nx workspace', (yargs) => yargs
    .option('name', {
    describe: chalk.dim `Workspace name (e.g. org name)`,
    type: 'string',
})
    .option('preset', {
    describe: chalk.dim `Customizes the initial content of your workspace. Default presets include: [${Object.values(preset_1.Preset)
        .map((p) => `"${p}"`)
        .join(', ')}]. To build your own see https://nx.dev/packages/nx-plugin#preset`,
    type: 'string',
})
    .option('appName', {
    describe: chalk.dim `The name of the application when a preset with pregenerated app is selected`,
    type: 'string',
})
    .option('interactive', {
    describe: chalk.dim `Enable interactive mode with presets`,
    type: 'boolean',
    default: true,
})
    .option('style', {
    describe: chalk.dim `Style option to be used when a preset with pregenerated app is selected`,
    type: 'string',
})
    .option('standaloneApi', {
    describe: chalk.dim `Use Standalone Components if generating an Angular app`,
    type: 'boolean',
})
    .option('routing', {
    describe: chalk.dim `Add a routing setup when a preset with pregenerated app is selected`,
    type: 'boolean',
})
    .option('bundler', {
    describe: chalk.dim `Bundler to be used to build the application`,
    choices: bundler_list_1.bundlerList,
    type: 'string',
})
    .option('framework', {
    describe: chalk.dim `Framework option to be used when the node-server preset is selected`,
    choices: framework_list_1.frameworkList,
    type: 'string',
})
    .option('docker', {
    describe: chalk.dim `Generate a Dockerfile with your node-server`,
    type: 'boolean',
})
    .option('nxCloud', {
    describe: chalk.dim(ab_testing_1.messages.getPromptMessage('nxCloudCreation')),
    type: 'boolean',
})
    .option('ci', {
    describe: chalk.dim `Generate a CI workflow file`,
    choices: ci_list_1.ciList,
    defaultDescription: '',
    type: 'string',
})
    .option('allPrompts', {
    alias: 'a',
    describe: chalk.dim `Show all prompts`,
    type: 'boolean',
    default: false,
})
    .option('packageManager', {
    alias: 'pm',
    describe: chalk.dim `Package manager to use`,
    choices: [...package_manager_1.packageManagerList].sort(),
    defaultDescription: 'npm',
    type: 'string',
})
    .option('defaultBase', {
    defaultDescription: 'main',
    describe: chalk.dim `Default base to use for new projects`,
    type: 'string',
})
    .option('skipGit', {
    describe: chalk.dim `Skip initializing a git repository`,
    type: 'boolean',
    default: false,
    alias: 'g',
})
    .option('commit.name', {
    describe: chalk.dim `Name of the committer`,
    type: 'string',
})
    .option('commit.email', {
    describe: chalk.dim `E-mail of the committer`,
    type: 'string',
})
    .option('commit.message', {
    describe: chalk.dim `Commit message`,
    type: 'string',
    default: 'Initial commit',
}), (argv) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    yield main(argv).catch((error) => {
        const { version } = require('../package.json');
        output_1.output.error({
            title: `Something went wrong! v${version}`,
        });
        throw error;
    });
}), [normalizeArgsMiddleware])
    .help('help', chalk.dim `Show help`)
    .updateLocale(decorator_1.yargsDecorator)
    .version('version', chalk.dim `Show version`, nx_version_1.nxVersion);
function main(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        yield (0, create_workspace_1.createWorkspace)(parsedArgs.preset, parsedArgs);
        if ((0, preset_1.isKnownPreset)(parsedArgs.preset)) {
            (0, point_to_tutorial_and_course_1.pointToTutorialAndCourse)(parsedArgs.preset);
        }
    });
}
/**
 * This function is used to normalize the arguments passed to the command.
 * It would:
 * - normalize the preset.
 * @param argv user arguments
 */
function normalizeArgsMiddleware(argv) {
    var _a, _b, _c, _d, _e, _f, _g;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        try {
            let name, appName, style, preset, framework, bundler, docker, routing, standaloneApi;
            output_1.output.log({
                title: "Let's create a new workspace [https://nx.dev/getting-started/intro]",
            });
            let thirdPartyPreset;
            try {
                thirdPartyPreset = yield (0, get_third_party_preset_1.getThirdPartyPreset)(argv.preset);
            }
            catch (e) {
                output_1.output.error({
                    title: `Could not find preset "${argv.preset}"`,
                });
                process.exit(1);
            }
            if (thirdPartyPreset) {
                preset = thirdPartyPreset;
                name = yield determineRepoName(argv);
                appName = '';
                style = null;
            }
            else {
                if (!argv.preset) {
                    const monorepoStyle = yield determineMonorepoStyle();
                    if (monorepoStyle === 'package-based') {
                        preset = 'npm';
                    }
                    else if (monorepoStyle === 'react') {
                        preset = preset_1.Preset.ReactStandalone;
                    }
                    else if (monorepoStyle === 'angular') {
                        preset = preset_1.Preset.AngularStandalone;
                    }
                    else if (monorepoStyle === 'node-server') {
                        preset = preset_1.Preset.NodeServer;
                    }
                    else {
                        preset = yield determinePreset(argv);
                    }
                }
                else if (argv.preset === 'react') {
                    preset = yield monorepoOrStandalone('react');
                }
                else if (argv.preset === 'angular') {
                    preset = yield monorepoOrStandalone('angular');
                }
                else {
                    preset = argv.preset;
                }
                if (preset === preset_1.Preset.ReactStandalone ||
                    preset === preset_1.Preset.AngularStandalone ||
                    preset === preset_1.Preset.NodeServer) {
                    appName =
                        (_b = (_a = argv.appName) !== null && _a !== void 0 ? _a : argv.name) !== null && _b !== void 0 ? _b : (yield determineAppName(preset, argv));
                    name = (_c = argv.name) !== null && _c !== void 0 ? _c : appName;
                    if (preset === preset_1.Preset.NodeServer) {
                        framework = yield determineFramework(argv);
                        docker = yield determineDockerfile(argv);
                    }
                    if (preset === preset_1.Preset.ReactStandalone) {
                        bundler = yield determineBundler(argv);
                    }
                    if (preset === preset_1.Preset.AngularStandalone) {
                        standaloneApi =
                            (_d = argv.standaloneApi) !== null && _d !== void 0 ? _d : (argv.interactive ? yield determineStandaloneApi(argv) : false);
                        routing =
                            (_e = argv.routing) !== null && _e !== void 0 ? _e : (argv.interactive ? yield determineRouting(argv) : true);
                    }
                }
                else {
                    name = yield determineRepoName(argv);
                    appName = yield determineAppName(preset, argv);
                    if (preset === preset_1.Preset.ReactMonorepo) {
                        bundler = yield determineBundler(argv);
                    }
                    if (preset === preset_1.Preset.AngularMonorepo) {
                        standaloneApi =
                            (_f = argv.standaloneApi) !== null && _f !== void 0 ? _f : (argv.interactive ? yield determineStandaloneApi(argv) : false);
                        routing =
                            (_g = argv.routing) !== null && _g !== void 0 ? _g : (argv.interactive ? yield determineRouting(argv) : true);
                    }
                }
                style = yield determineStyle(preset, argv);
            }
            const packageManager = yield determinePackageManager(argv);
            const defaultBase = yield determineDefaultBase(argv);
            const nxCloud = yield determineNxCloud(argv);
            const ci = yield determineCI(argv, nxCloud);
            Object.assign(argv, {
                name,
                preset,
                appName,
                style,
                standaloneApi,
                routing,
                framework,
                nxCloud,
                packageManager,
                defaultBase,
                ci,
                bundler,
                docker,
            });
        }
        catch (e) {
            console.error(e);
            process.exit(1);
        }
    });
}
function determineRepoName(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const repoName = parsedArgs._[0]
            ? parsedArgs._[0].toString()
            : parsedArgs.name;
        if (repoName) {
            return Promise.resolve(repoName);
        }
        const a = yield enquirer.prompt([
            {
                name: 'RepoName',
                message: `Repository name                      `,
                type: 'input',
            },
        ]);
        if (!a.RepoName) {
            output_1.output.error({
                title: 'Invalid repository name',
                bodyLines: [`Repository name cannot be empty`],
            });
            process.exit(1);
        }
        return a.RepoName;
    });
}
function monorepoOrStandalone(preset) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const a = yield enquirer.prompt([
            {
                name: 'MonorepoOrStandalone',
                message: `--preset=${preset} has been replaced with the following:`,
                type: 'autocomplete',
                choices: [
                    {
                        name: preset + '-standalone',
                        message: `${preset}-standalone: a standalone ${preset} application.`,
                    },
                    {
                        name: preset + '-monorepo',
                        message: `${preset}-monorepo:   a monorepo with the apps and libs folders.`,
                    },
                ],
            },
        ]);
        if (!a.MonorepoOrStandalone) {
            output_1.output.error({
                title: 'Invalid selection',
            });
            process.exit(1);
        }
        return a.MonorepoOrStandalone;
    });
}
function determineMonorepoStyle() {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const a = yield enquirer.prompt([
            {
                name: 'MonorepoStyle',
                message: `Choose what to create                `,
                type: 'autocomplete',
                choices: [
                    {
                        name: 'package-based',
                        message: 'Package-based monorepo:     Nx makes it fast, but lets you run things your way.',
                    },
                    {
                        name: 'integrated',
                        message: 'Integrated monorepo:        Nx configures your favorite frameworks and lets you focus on shipping features.',
                    },
                    {
                        name: 'react',
                        message: 'Standalone React app:       Nx configures Vite (or Webpack), ESLint, and Cypress.',
                    },
                    {
                        name: 'angular',
                        message: 'Standalone Angular app:     Nx configures Jest, ESLint and Cypress.',
                    },
                    {
                        name: 'node-server',
                        message: 'Standalone Node Server app: Nx configures a framework (ex. Express), esbuild, ESlint and Jest.',
                    },
                ],
            },
        ]);
        if (!a.MonorepoStyle) {
            output_1.output.error({
                title: 'Invalid monorepo style',
            });
            process.exit(1);
        }
        return a.MonorepoStyle;
    });
}
function determinePackageManager(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const packageManager = parsedArgs.packageManager;
        if (packageManager) {
            if (package_manager_1.packageManagerList.includes(packageManager)) {
                return Promise.resolve(packageManager);
            }
            output_1.output.error({
                title: 'Invalid package manager',
                bodyLines: [
                    `Package manager must be one of ${(0, string_utils_1.stringifyCollection)([
                        ...package_manager_1.packageManagerList,
                    ])}`,
                ],
            });
            process.exit(1);
        }
        if (parsedArgs.allPrompts) {
            return enquirer
                .prompt([
                {
                    name: 'packageManager',
                    message: `Which package manager to use         `,
                    initial: 'npm',
                    type: 'autocomplete',
                    choices: [
                        { name: 'npm', message: 'NPM' },
                        { name: 'yarn', message: 'Yarn' },
                        { name: 'pnpm', message: 'PNPM' },
                    ],
                },
            ])
                .then((a) => a.packageManager);
        }
        return Promise.resolve((0, package_manager_1.detectInvokedPackageManager)());
    });
}
function determineDefaultBase(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.defaultBase) {
            return Promise.resolve(parsedArgs.defaultBase);
        }
        if (parsedArgs.allPrompts) {
            return enquirer
                .prompt([
                {
                    name: 'DefaultBase',
                    message: `Main branch name                   `,
                    initial: `main`,
                    type: 'input',
                },
            ])
                .then((a) => {
                if (!a.DefaultBase) {
                    output_1.output.error({
                        title: 'Invalid branch name',
                        bodyLines: [`Branch name cannot be empty`],
                    });
                    process.exit(1);
                }
                return a.DefaultBase;
            });
        }
        return Promise.resolve((0, default_base_1.deduceDefaultBase)());
    });
}
function determinePreset(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.preset) {
            if (Object.values(preset_1.Preset).indexOf(parsedArgs.preset) === -1) {
                output_1.output.error({
                    title: 'Invalid preset',
                    bodyLines: [
                        `It must be one of the following:`,
                        '',
                        ...Object.values(preset_1.Preset),
                    ],
                });
                process.exit(1);
            }
            else {
                return Promise.resolve(parsedArgs.preset);
            }
        }
        return enquirer
            .prompt([
            {
                name: 'preset',
                message: `What to create in the new workspace  `,
                initial: 'empty',
                type: 'autocomplete',
                choices: preset_options_1.presetOptions,
            },
        ])
            .then((a) => a.preset);
    });
}
function determineAppName(preset, parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (preset === preset_1.Preset.Apps ||
            preset === preset_1.Preset.Core ||
            preset === preset_1.Preset.TS ||
            preset === preset_1.Preset.Empty ||
            preset === preset_1.Preset.NPM) {
            return Promise.resolve('');
        }
        if (parsedArgs.appName) {
            return Promise.resolve(parsedArgs.appName);
        }
        return enquirer
            .prompt([
            {
                name: 'AppName',
                message: `Application name                     `,
                type: 'input',
            },
        ])
            .then((a) => {
            if (!a.AppName) {
                output_1.output.error({
                    title: 'Invalid name',
                    bodyLines: [`Name cannot be empty`],
                });
                process.exit(1);
            }
            return a.AppName;
        });
    });
}
function determineFramework(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const frameworkChoices = [
            {
                name: 'express',
                message: 'Express [https://expressjs.com/]',
            },
            {
                name: 'fastify',
                message: 'Fastify [https://www.fastify.io/]',
            },
            {
                name: 'koa',
                message: 'Koa     [https://koajs.com/]',
            },
            {
                name: 'nest',
                message: 'NestJs  [https://nestjs.com/]',
            },
        ];
        if (!parsedArgs.framework) {
            return enquirer
                .prompt([
                {
                    message: 'What framework should be used?',
                    type: 'autocomplete',
                    name: 'framework',
                    choices: frameworkChoices,
                },
            ])
                .then((a) => a.framework);
        }
        const foundFramework = frameworkChoices
            .map(({ name }) => name)
            .indexOf(parsedArgs.framework);
        if (foundFramework < 0) {
            output_1.output.error({
                title: 'Invalid framework',
                bodyLines: [
                    `It must be one of the following:`,
                    '',
                    ...frameworkChoices.map(({ name }) => name),
                ],
            });
            process.exit(1);
        }
        return Promise.resolve(parsedArgs.framework);
    });
}
function determineStandaloneApi(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.standaloneApi === undefined) {
            return enquirer
                .prompt([
                {
                    name: 'standaloneApi',
                    message: 'Would you like to use Standalone Components in your application?',
                    type: 'autocomplete',
                    choices: [
                        {
                            name: 'Yes',
                        },
                        {
                            name: 'No',
                        },
                    ],
                    initial: 'No',
                },
            ])
                .then((a) => a.standaloneApi === 'Yes');
        }
        return parsedArgs.standaloneApi;
    });
}
function determineDockerfile(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.docker === undefined) {
            return enquirer
                .prompt([
                {
                    name: 'docker',
                    message: 'Would you like to generate a Dockerfile? [https://docs.docker.com/]',
                    type: 'autocomplete',
                    choices: [
                        {
                            name: 'Yes',
                            hint: 'I want to generate a Dockerfile',
                        },
                        {
                            name: 'No',
                        },
                    ],
                    initial: 'No',
                },
            ])
                .then((a) => a.docker === 'Yes');
        }
        else {
            return Promise.resolve(parsedArgs.docker);
        }
    });
}
function determineStyle(preset, parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (preset === preset_1.Preset.Apps ||
            preset === preset_1.Preset.Core ||
            preset === preset_1.Preset.TS ||
            preset === preset_1.Preset.Empty ||
            preset === preset_1.Preset.NPM ||
            preset === preset_1.Preset.Nest ||
            preset === preset_1.Preset.Express ||
            preset === preset_1.Preset.ReactNative ||
            preset === preset_1.Preset.Expo ||
            preset === preset_1.Preset.NodeServer) {
            return Promise.resolve(null);
        }
        const choices = [
            {
                name: 'css',
                message: 'CSS',
            },
            {
                name: 'scss',
                message: 'SASS(.scss)       [ http://sass-lang.com   ]',
            },
            {
                name: 'less',
                message: 'LESS              [ http://lesscss.org     ]',
            },
        ];
        if (![preset_1.Preset.AngularMonorepo, preset_1.Preset.AngularStandalone].includes(preset)) {
            choices.push({
                name: 'styl',
                message: 'Stylus(.styl)     [ http://stylus-lang.com ]',
            });
        }
        if ([preset_1.Preset.ReactMonorepo, preset_1.Preset.ReactStandalone, preset_1.Preset.NextJs].includes(preset)) {
            choices.push({
                name: 'styled-components',
                message: 'styled-components [ https://styled-components.com            ]',
            }, {
                name: '@emotion/styled',
                message: 'emotion           [ https://emotion.sh                       ]',
            }, {
                name: 'styled-jsx',
                message: 'styled-jsx        [ https://www.npmjs.com/package/styled-jsx ]',
            });
        }
        if (!parsedArgs.style) {
            return enquirer
                .prompt([
                {
                    name: 'style',
                    message: `Default stylesheet format            `,
                    initial: 'css',
                    type: 'autocomplete',
                    choices: choices,
                },
            ])
                .then((a) => a.style);
        }
        const foundStyle = choices.find((choice) => choice.name === parsedArgs.style);
        if (foundStyle === undefined) {
            output_1.output.error({
                title: 'Invalid style',
                bodyLines: [
                    `It must be one of the following:`,
                    '',
                    ...choices.map((choice) => choice.name),
                ],
            });
            process.exit(1);
        }
        return Promise.resolve(parsedArgs.style);
    });
}
function determineRouting(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!parsedArgs.routing) {
            return enquirer
                .prompt([
                {
                    name: 'routing',
                    message: 'Would you like to add routing?',
                    type: 'autocomplete',
                    choices: [
                        {
                            name: 'Yes',
                        },
                        {
                            name: 'No',
                        },
                    ],
                    initial: 'Yes',
                },
            ])
                .then((a) => a.routing === 'Yes');
        }
        return parsedArgs.routing;
    });
}
function determineBundler(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const choices = [
            {
                name: 'vite',
                message: 'Vite    [ https://vitejs.dev/ ]',
            },
            {
                name: 'webpack',
                message: 'Webpack [ https://webpack.js.org/ ]',
            },
        ];
        if (!parsedArgs.bundler) {
            return enquirer
                .prompt([
                {
                    name: 'bundler',
                    message: `Bundler to be used to build the application`,
                    initial: 'vite',
                    type: 'autocomplete',
                    choices: choices,
                },
            ])
                .then((a) => a.bundler);
        }
        const foundBundler = choices.find((choice) => choice.name === parsedArgs.bundler);
        if (foundBundler === undefined) {
            output_1.output.error({
                title: 'Invalid bundler',
                bodyLines: [
                    `It must be one of the following:`,
                    '',
                    ...choices.map((choice) => choice.name),
                ],
            });
            process.exit(1);
        }
        return Promise.resolve(parsedArgs.bundler);
    });
}
function determineNxCloud(parsedArgs) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (parsedArgs.nxCloud === undefined) {
            return enquirer
                .prompt([
                {
                    name: 'NxCloud',
                    message: ab_testing_1.messages.getPromptMessage('nxCloudCreation'),
                    type: 'autocomplete',
                    choices: [
                        {
                            name: 'Yes',
                            hint: 'I want faster builds',
                        },
                        {
                            name: 'No',
                        },
                    ],
                    initial: 'Yes',
                },
            ])
                .then((a) => a.NxCloud === 'Yes');
        }
        else {
            return parsedArgs.nxCloud;
        }
    });
}
function determineCI(parsedArgs, nxCloud) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!nxCloud) {
            if (parsedArgs.ci) {
                output_1.output.warn({
                    title: 'Invalid CI value',
                    bodyLines: [
                        `CI option only works when Nx Cloud is enabled.`,
                        `The value provided will be ignored.`,
                    ],
                });
            }
            return '';
        }
        if (parsedArgs.ci) {
            return parsedArgs.ci;
        }
        if (parsedArgs.allPrompts) {
            return (enquirer
                .prompt([
                {
                    name: 'CI',
                    message: `CI workflow file to generate?      `,
                    type: 'autocomplete',
                    initial: '',
                    choices: [
                        { message: 'none', name: '' },
                        { message: 'GitHub Actions', name: 'github' },
                        { message: 'Circle CI', name: 'circleci' },
                        { message: 'Azure DevOps', name: 'azure' },
                    ],
                },
            ])
                // enquirer ignores name and value if they are falsy and takes
                // first field that has a truthy value, so wee need to explicitly
                // check for none
                .then((a) => (a.CI !== 'none' ? a.CI : '')));
        }
        return '';
    });
}
//# sourceMappingURL=create-nx-workspace.js.map